#pragma once

#include <filesystem>
#include <map>
#include <string>

#include <SimoxUtility/json.h>

#include <ArmarXCore/core/services/tasks/PeriodicTask.h>

#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>

#include "util/filesystem.h"

namespace armarx::armem::server::ltm::detail::mixin
{
    using Path = std::filesystem::path;

    class DiskMemoryItemMixin
    {
    public:
        DiskMemoryItemMixin() = default;
        DiskMemoryItemMixin(const Path& memoryParentPath,
                            const std::string& exportName,
                            const armem::MemoryID& id);
        virtual ~DiskMemoryItemMixin() = default;

        Path getMemoryBasePath() const;
        Path getFullPath() const;

        // Filesystem interaction
        bool memoryBasePathExists() const;
        bool fullPathExists() const;
        bool fileExists(const std::string& filename) const;

        void ensureMemoryBasePathExists(bool createIfNotExistent = false) const;
        void ensureFullPathExists(bool createIfNotExistent = false) const;
        void ensureFileExists(const std::string& filename, bool createIfNotExistent = false) const;

        void writeDataToFile(const std::string& filename,
                             const std::vector<unsigned char>& data) const;

        std::vector<unsigned char> readDataFromFile(const std::string& filename) const;
        std::vector<Path> getAllDirectories() const;
        std::vector<Path> getAllFiles() const;


    protected:
        // setter
        void setMixinMemoryID(const MemoryID& n);
        void setMemoryBasePath(const std::filesystem::path& n);
        void setMixinExportName(const std::string& n);

        /// configuration
        void configureMixin(const nlohmann::json& json);

    public:
        static const int DEPTH_TO_DATA_FILES =
            7; // from memory folder = 1 (cseg) + 1 (pseg) + 1 (ent) + 3 (snap) + 1 (inst)

    protected:
        static const constexpr char* TYPE_FILENAME = "type.aron";
        static const constexpr char* DATA_FILENAME = "data.aron";
        static const constexpr char* METADATA_FILENAME = "metadata.aron";

        static const constexpr char* MEMORY_EXPORT_SUFFIX = "_";

    private:
        std::filesystem::path memoryBasePath;
        std::string exportName;
        armem::MemoryID _id;
    };
} // namespace armarx::armem::server::ltm::detail::mixin
