#pragma once

// STD/STL
#include <memory>

// Simox
#include <SimoxUtility/json.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>

#include "../../extractor/Extractor.h"

namespace armarx::armem::server::ltm::processor
{
    class DataConverter
    {
    public:
        enum class ConverterType
        {
            Str,
            Binary
        };

        struct ConversionResult
        {
            std::vector<unsigned char> data;
            std::string suffix;
        };

        DataConverter(const ConverterType t,
                      const std::string& id,
                      const std::string& s,
                      const aron::type::Descriptor c,
                      std::unique_ptr<Extractor>&& ex) :
            type(t), identifier(id), suffix(s), convertsType(c), extractor(std::move(ex))
        {
        }

        virtual ~DataConverter() = default;

        virtual ConversionResult convert(const aron::data::VariantPtr& data) = 0;
        virtual aron::data::VariantPtr convert(const ConversionResult&,
                                               const armarx::aron::Path& p) = 0;

        virtual void configure(const nlohmann::json& json);

    public:
        const ConverterType type;
        const std::string identifier;
        const std::string suffix;
        const aron::type::Descriptor convertsType;

        const std::unique_ptr<Extractor> extractor;
    };
} // namespace armarx::armem::server::ltm::processor
